--[[
==============
==============          LFRSatchelHelper
==============    An addon by Lyrenia - Draenor
==============  Scan LFR and dungeons for satchels
==============
--]]

local LFRSH = LFRSatchelHelperGlobal

LFRSH.timeSinceLastScan = 0
LFRSH.autoScan = false
LFRSH.chatMsgPrinted = false                    -- prevents spam in chat window 
LFRSH.lastQueuedInstance = nil					-- holds instance type of last instance we queued for				
LFRSH.isManualScan = false

local addonVersion = "1.0.1"
local totalSatchelsFound = 0                    -- store the amount of satchels we found in a scan
local scanResultLFRMsg = ""                     -- holds the chat message for LFR
local scanResultDungeonMsg = ""                 -- holds the chat message for dungeons
local previousScanResultLFR = {}                -- holds previous scan results for LFR to compare with new results
local previousScanResultDungeons = {}           -- holds previous scan results for dungeons to compare with new results

-- handle the queue button press
function LFRSH.HandleQueueing(instanceType, id)
    local instanceID, qL, qT, qH, qD

    if instanceType == "LFR" then
        instanceID = LFRSH.lfrIDs[id][2]
        qL = _G["LFRSH_RaidRow"..id.."LeaderCB"]:GetChecked()
        qT = _G["LFRSH_RaidRow"..id.."TankCB"]:GetChecked()
        qH = _G["LFRSH_RaidRow"..id.."HealerCB"]:GetChecked()
        qD = _G["LFRSH_RaidRow"..id.."DamageCB"]:GetChecked()
    elseif instanceType == "Dungeons" then
        instanceID = LFRSH.dungeonIDs[id][2]
        qL = _G["LFRSH_DungeonRow"..id.."LeaderCB"]:GetChecked()
        qT = _G["LFRSH_DungeonRow"..id.."TankCB"]:GetChecked()
        qH = _G["LFRSH_DungeonRow"..id.."HealerCB"]:GetChecked()
        qD = _G["LFRSH_DungeonRow"..id.."DamageCB"]:GetChecked()
    end
    
    -- set the roles for this instance
    SetLFGRoles(qL, qT, qH, qD)
    -- clear previously queued for raids
    ClearAllLFGDungeons(LE_LFG_CATEGORY_LFR)
    -- set the new raid for queueing
    SetLFGDungeon(LE_LFG_CATEGORY_LFR, instanceID)
    -- queue for the raid
    JoinLFG(LE_LFG_CATEGORY_LFR)
end

-- search through the available lfr instances and check if they have satchel rewards
function LFRSH.satchelFinder(id, instanceType)
    local eligible, forTank, forHealer, forDamage = GetLFGRoleShortageRewards(id, 1)
    local lfrInstanceName = GetLFGDungeonInfo(id)
    local doneThisWeek, money = GetLFGDungeonRewards(id)
    local rolesNeededMsg = ""

    if forTank and LFRSH.canTank then
        -- show tank in white (used to be blue but unreadable)   
        rolesNeededMsg = rolesNeededMsg .. " \124cFFFFFFFFTANK\124r"
    end
    if forHealer and LFRSH.canHeal then
        -- show healer in green
        rolesNeededMsg = rolesNeededMsg .. " \124cFF00FF00HEALER\124r"   
    end
    if forDamage and LFRSH.canDPS then
        -- show damage in red  
        rolesNeededMsg = rolesNeededMsg .. " \124cFFFF0000DAMAGE\124r"
    end
       
    if forTank or forHealer or forDamage then
        local doneThisWeekMsg = ""
        if not doneThisWeek then
            if instanceType == "LFR" then
                doneThisWeekMsg = "\124c0000FFFF Weekly Reward! \124r"
            elseif instanceType == "Dungeons" then
                doneThisWeekMsg = "\124c0000FFFF Daily Reward! \124r"
            end
        end
        totalSatchelsFound = totalSatchelsFound + 1
        if instanceType == "LFR" then
            scanResultLFRMsg = scanResultLFRMsg .. "\124cFFFFFF00[LFRSH] " .. lfrInstanceName .. ":" .. doneThisWeekMsg .. rolesNeededMsg .. "\n"
        elseif instanceType == "Dungeons" then
            scanResultDungeonMsg = scanResultDungeonMsg .. "\124cFFFFFF00[LFRSH] " .. lfrInstanceName .. ":" .. doneThisWeekMsg .. rolesNeededMsg .. "\n"
        end
    end
    return {forTank, forHealer, forDamage}, doneThisWeek
end

-- simple loop that calls the actual scan function
function LFRSH.Scan()
        --scanResultLFRMsg = "Results:\n"
        --scanResultDungeonMsg = "Results:\n"        
		scanResultLFRMsg = ""
        scanResultDungeonMsg = ""
        local scanResult, weekly, scanResultDungeon, daily = {}, {}, {}, {}

        -- scan lfr
        for i=1, #LFRSH.lfrIDs do
            scanResult[i], weekly[i] = LFRSH.satchelFinder(LFRSH.lfrIDs[i][2], "LFR")
        end
        -- scan heroic and tw
        for j=1, #LFRSH.dungeonIDs do
            scanResultDungeon[j], daily[j] = LFRSH.satchelFinder(LFRSH.dungeonIDs[j][2], "Dungeons")
        end

        if totalSatchelsFound == 0 then
            -- no point in spamming chat when nothing is found while autoscan is up
            if not LFRSH.autoScan then 
                LFRSH.print("No satchels found")
                -- update weekly and daily rewards in ui
                LFRSH.UpdateLFRGUI(nil, weekly)
                LFRSH.UpdateDungeonGUI(nil, daily)
            end
        else
			local notify = false;
            if not LFRSH.CompareTables(previousScanResultLFR, scanResult) or LFRSH.isManualScan then
                LFRSH.print("LFR scanned at: " .. date("%H:%M:%S"))
                if scanResultLFRMsg == "" then
					LFRSH.print("No results")
				else
					LFRSH.print(scanResultLFRMsg, true)
                end
				previousScanResultLFR = scanResult
                -- mark checkboxes for the roles
                LFRSH.UpdateLFRGUI(scanResult, weekly)
				notify = true;
            else
                --LFRSH.print("Same results as last scan")
            end

            if not LFRSH.CompareTables(previousScanResultDungeons, scanResultDungeon) or LFRSH.isManualScan then
                LFRSH.print("Dungeons scanned at: " .. date("%H:%M:%S"))
                if scanResultDungeonMsg == "" then
					LFRSH.print("No results")
				else
					LFRSH.print(scanResultDungeonMsg, true)
                end
				previousScanResultDungeons = scanResultDungeon
                -- mark checkboxes for the roles
                LFRSH.UpdateDungeonGUI(scanResultDungeon, daily)
				notify = true;
            else
                --LFRSH.print("Same results as last scan")
            end
			
			if notify then
				local mergedResult = LFRSH.Copy(scanResult)
				for i = 1, #scanResultDungeon do
					mergedResult[#mergedResult + i] = scanResultDungeon[i]
				end
                -- send notifications
				LFRSH.CheckNotifications(mergedResult)
				mergedResult = nil
			end
			
        end
        totalSatchelsFound = 0
		LFRSH.isManualScan = false
end

-- On Load
function LFRSatchelHelper_OnLoad(self)
	print("\124cFFFFFF00[LFRSH] version " .. addonVersion .. " loaded\124r")
   -- print("\124cFFFFFF00[LFRSH] '/lfrsh scan' or click minimap button to look for LFRs/Dungeons that reward a satchel!\124r")
   -- print("\124cFFFFFF00[LFRSH] '/lfrsh autoscan' or right click minimap button to scan every second (default)\124r")
   -- print("\124cFFFFFF00[LFRSH] shift click minimap button to show gui\124r")
    self:RegisterEvent("ADDON_LOADED")
    self:RegisterEvent("LFG_UPDATE_RANDOM_INFO")
    self:RegisterEvent("LFG_COMPLETION_REWARD")
    self:RegisterEvent("UI_INFO_MESSAGE")
    self:RegisterEvent("LFG_UPDATE")
    self:SetScript("OnEvent", LFRSatchelHelper_OnEvent)

    -- look up the roles the player can perform (only what technically can be done, not what specs were learned)
    local canTank, canHeal, canDPS = UnitGetAvailableRoles("player")
    LFRSH.canTank = canTank
    LFRSH.canHeal = canHeal
    LFRSH.canDPS = canDPS

    -- create ui
    LFRSH.panel = LFRSH.CreateUI()

    -- Register tabs with blizz code    
    PanelTemplates_SetNumTabs(LFRSH.panel, 3)
    PanelTemplates_SetTab(LFRSH.panel, 1)
	_G["LFRSH_Tab1"]:Show()
	
    -- register slash commands
    SLASH_LFRSATCHELHELPER1 = "/lfrsh"
    SlashCmdList["LFRSATCHELHELPER"] = function(msg)
        if msg == "scan" then           
            LFRSH.chatMsgPrinted = false
            LFRSH.isManualScan = true
			-- force the update random info event (to get the latest information) after which we will do our scan
            RequestLFDPlayerLockInfo()    
        elseif msg == "autoscan" then
            if LFRSH.autoScan then
				LFRSH.autoScan = false
				LFRSH.print("Autscan disabled")
				SetDesaturation(_G["LFRSatchelHelper_MinimapButton_Icon"], false)				
			else
				LFRSH.autoScan = true
				LFRSH.timeSinceLastScan = 0
				LFRSH.print("Autoscan enabled")			
			end
        elseif msg == "gui" then
			if LFRSH.panel:IsShown() then
				LFRSH.panel:Hide()
			else
				LFRSH.panel:Show()
			end
        elseif msg == "minimap" then   
			if _G["LFRSatchelHelper_MinimapButton"]:IsShown() then
				_G["LFRSatchelHelper_MinimapButton"]:Hide()
				LFRSatchelHelperSettings["ShowMinimapButton"] = false
			else
				_G["LFRSatchelHelper_MinimapButton"]:Show()
				LFRSatchelHelperSettings["ShowMinimapButton"] = true
			end
		else -- unknown command, list the available commands
			LFRSH.print("List of available commands:" ..
			"\n\124c0000FFFFscan\124r: scan for satchels (or click on minimap button)" ..
			"\n\124c0000FFFFautoscan\124r: toggle autoscan (or right click minimap button)" ..
			"\n\124c0000FFFFgui\124r: toggle gui (or shift click on minimap button)" ..
			"\n\124c0000FFFFminimap\124r: toggle minimapbutton");
        end
    end 
end

local function LoadSettings()
	
	if LFRSatchelHelperSettings["ShowMinimapButton"] then
		_G["LFRSatchelHelper_MinimapButton"]:Show()
	else
		_G["LFRSatchelHelper_MinimapButton"]:Hide()
	end	
	
	if LFRSatchelHelperSettings["ShowChatMessages"] then
		_G["LFRSH_Tab3_Body_ShowChatCheckButton"]:SetChecked(true)
	else
		_G["LFRSH_Tab3_Body_ShowChatCheckButton"]:SetChecked(false)
	end
	if LFRSatchelHelperSettings["NotifySound"] then
		_G["LFRSH_Tab3_Body_NotifySoundCheckButton"]:SetChecked(true)
	else
		_G["LFRSH_Tab3_Body_NotifySoundCheckButton"]:SetChecked(false)
	end	
	_G["LFRSH_Tab3_Body_ScanIntervalEditBox"]:SetText(tostring(LFRSatchelHelperSettings["ScanInterval"]))

	LFRSH.UpdateNotificationsFrame()
	
end

-- eventhandler
function LFRSatchelHelper_OnEvent(self, event, ...)
    if event == "ADDON_LOADED" and select(1, ...) == "LFRSatchelHelper" then
		-- Init settings for the first time
        if LFRSatchelHelperSettings == nil then
            LFRSatchelHelperSettings = {
                ["MinimapButtonPos"] 	= 45,
				["ShowMinimapButton"] 	= true,				
				["ShowChatMessages"] 	= true,
				["ScanInterval"] 		= 1,
				["NotifySound"] 		= true,
				["NotifyFlash"] 		= true,
				["NotifyPopup"] 		= true,
				["NotifyInstances"]		= {}
            }
			for i=1, #LFRSH.quickLookupIDs do
				for instanceID,_ in pairs(LFRSH.quickLookupIDs[i]) do
					LFRSatchelHelperSettings["NotifyInstances"][instanceID] = {false, false, false} -- bTank, bHealer, bDamage
				end
			end
        end
		
		LoadSettings()
		
        -- put minimap button in the correct place
        LFRSH.MinimapReposition()

        -- reset the time since the last auto scan
        LFRSH.timeSinceLastScan = 0
        -- init prev scan result
        for i = 1, #LFRSH.lfrIDs do
            previousScanResultLFR[i] = {false, false, false}
        end
        for j = 1, #LFRSH.dungeonIDs do
            previousScanResultDungeons[j] = {false, false, false}
        end

    end

    if event == "LFG_UPDATE_RANDOM_INFO" and not LFRSH.chatMsgPrinted then
            LFRSH.Scan()
            LFRSH.chatMsgPrinted = true			
    end
	
	if event == "LFG_UPDATE" then
		LFRSH.ToggleQueueButtons()
	end
	
	if event == "LFG_COMPLETION_REWARD" then
		-- track rewards/time spend from lfr/dungeons for statistics
		local name, typeID, subtypeID, textureFilename, moneyBase, moneyVar, experienceBase, experienceVar, numStrangers, numRewards = GetLFGCompletionReward();
		--print(name, typeID, subtypeID, textureFilename, moneyBase, moneyVar, experienceBase, experienceVar, numStrangers, numRewards)
	end	
end

-- On update so we can do stuff with intervals in seconds
function LFRSatchelHelper_OnUpdate(self, elapsed)
    if LFRSH.autoScan then
        LFRSH.timeSinceLastScan = LFRSH.timeSinceLastScan + elapsed
        if LFRSH.timeSinceLastScan > LFRSatchelHelperSettings["ScanInterval"] then
			-- trigger a scan
            LFRSH.chatMsgPrinted = false
            RequestLFDPlayerLockInfo()
            -- reset time
            LFRSH.timeSinceLastScan = 0
        end
    end	
end

-- make it close when the user presses ESC
tinsert(UISpecialFrames, "LFRSatchelHelperFrame")