--[[ 
==============
==============    Helper functions and variables
==============
--]]

-- Global variable visible to the rest of WOW
LFRSatchelHelperGlobal = {}
local LFRSH = LFRSatchelHelperGlobal
--[[
	both of the following arrays are used when we want this exact order when used in a loop
	sadly we can't use an array like quickLookupIDs because when we loop through it we have to use in pairs() 
	and that gives us no control over the order of the items
--]]
-- array to specify available LFR instances NAME , INSTANCE_ID
LFRSH.lfrIDs = {
    {"Walled City"          , 849}, 
    {"Arcane Sanctum"       , 850}, 
    {"Imperator's Rise"     , 851}, 
    {"Slagworks"            , 847}, 
    {"The Black Forge"      , 846}, 
    {"Iron Assembly"        , 848}, 
    {"Blackhand's Crucible" , 823}, 
    {"Hellbreach"           , 982}, 
    {"Halls of Blood"       , 983}, 
    {"Bastion of Shadows"   , 984}, 
    {"Destructor's Rise"    , 985}, 
    {"The Black Gate"       , 986} 
}

-- array to specify available dungeons NAME, INSTANCE_ID, IS_TIMEWARPED
LFRSH.dungeonIDs = {
    {"Heroic WOD"          , 789    , false},
    {"TW Lich King"        , 995    , true},
    {"TW Burning Crusade"  , 744    , true},
    {"TW Cataclysm"        , 1146   , true}
}

-- array used to lookup instances with their ID, much quicker than cycling through for ex lfrIDs
LFRSH.quickLookupIDs = {
    -- lfr
	{
		[849] = {1, "Walled City"}, 
		[850] = {2, "Arcane Sanctum"}, 
		[851] = {3, "Imperator's Rise"},
		[847] = {4, "Slagworks"},
		[846] = {5, "The Black Forge"},
		[848] = {6, "Iron Assembly"},
		[823] = {7, "Blackhand's Crucible"},
		[982] = {8, "Hellbreach"},
		[983] = {9, "Halls of Blood"},
		[984] = {10, "Bastion of Shadows"},
		[985] = {11, "Destructor's Rise"},
		[986] = {12, "The Black Gate"}
	},
	-- dungeons
	{
		[789] = 	{1, "Heroic WOD", true},
		[995] = 	{2, "TW Lich King", true},
		[744] = 	{3, "TW Burning Crusade", true},
		[1146] = 	{4, "TW Cataclysm", true}
	}
}

-- array that holds the available roles and the color we want for the text when the role name is shown somewhere 
LFRSH.quickLookupRoles = {
	{"Tank", "FFFFFFFF"},
	{"Healer", "FF00FF00"}, 
	{"Damage", "FFFF0000"}
}

-- usefull when you don't know the instance type
function LFRSH.QuickLookup(instanceID)
	for i=1, #LFRSH.quickLookupIDs do
		if LFRSH.quickLookupIDs[i][instanceID] ~= nil then
			return LFRSH.quickLookupIDs[i][instanceID]
		end
	end
	return nil
end

-- Print a message in chat with [LFRSH] in front in yellow
function LFRSH.print(m, useNormalPrint)
    if LFRSatchelHelperSettings["ShowChatMessages"] then
		if not useNormalPrint then
			print("\124cFFFFFF00[LFRSH] " .. m .. "\124r")
		else
			print(m)
		end
	end
end

-- simple comparison function for two arrays. MUST BE SAME DIMENSIONS
function LFRSH.CompareTables(arrayA, arrayB)
   for i = 1, #arrayA do
      for j = 1, #arrayA[i] do
         if arrayA[i][j] ~= arrayB[i][j] then
            return false
         end       
      end
   end
   return #arrayA == #arrayB
end

-- convert seconds to hours, mins, seconds string
function LFRSH.FormatTime(totalSeconds)
	if totalSeconds <= 0 then 
		return "0h 0m 0s" 
	else
		local hours 	= math.floor(totalSeconds / 3600)
		local mins 		= math.floor(totalSeconds / 60 - (hours * 60)) 
		local seconds 	= math.floor(totalSeconds - hours * 3600 - mins * 60)
		return string.format("%dh %dm %ds", hours, mins, seconds)
	end
end

function LFRSH.CheckNotifications(scanResult)
	local notifications = {}
	local notificationNeeded = false
	for instanceID, roles in pairs(LFRSatchelHelperSettings["NotifyInstances"]) do
		for i=1, #roles do
			local instanceData = LFRSH.QuickLookup(instanceID)
			if roles[i] then
				local scanInstanceID = instanceData[3] == true and instanceData[1]+12 or instanceData[1] 
				if scanResult[scanInstanceID][i] then
					-- don't play sound/show popup and what not when we're already in queue for this instance 
					if not LFRSH.InQueue(instanceID) then
						notificationNeeded = true
					end
					print("\124cFF00FF00[LFRSH] " .. LFRSH.quickLookupRoles[i][1] .. " in " .. instanceData[2] .. "\124r")
					--print(i, roles[i], instanceID, instanceData[1], scanResult[instanceData[1]][i], instanceData[2])
					--notifications[instanceID][] = {LFRSH.quickLookupRoles[i], instanceData}
				end
			end
		end
	end
	if notificationNeeded then
		LFRSH.SendNotification(notifications)
	end
end

function LFRSH.SendNotification(notifications)
	
	-- flash the wow icon
	if LFRSatchelHelperSettings["NotifyFlash"] then
		FlashClientIcon()
	end
	-- play sound file
	if LFRSatchelHelperSettings["NotifySound"] then
		PlaySound("LEVELUPSOUND", "Master")
	end
	-- show popup 
--	for instanceID, roles in pairs(notifications) do
		
--	end
	--LFRSH.NotificationPopup.instanceLabel:SetText(instanceData[2])
	--LFRSH.NotificationPopup.rolesLabel:SetText(LFRSH.quickLookupRoles[roleID][1])
	--LFRSH.NotificationPopup:Show()
end

-- add a notification to the settings table
function LFRSH.AddNotification(instanceID, roleID)
	if instanceID and roleID then
		LFRSatchelHelperSettings["NotifyInstances"][instanceID][roleID] = true
		LFRSH.UpdateNotificationsFrame()
	end
end

-- remove a notification from the settings table
function LFRSH.RemoveNotification(instanceID, roleID)
	if instanceID and roleID then
		LFRSatchelHelperSettings["NotifyInstances"][instanceID][roleID] = false
		LFRSH.UpdateNotificationsFrame()
	end
end

function LFRSH.InQueue(instanceID)
	if instanceID then
		for id, isQueued in pairs(LFGQueuedForList[3]) do
			if instanceID == id then
				return true
			end
		end
		for id, isQueued in pairs(LFGQueuedForList[1]) do
			if instanceID == id then
				return true
			end
		end	
		return false		
	else
		return false
	end
end

function LFRSH.Copy(array)
	local arrCopy = {}
	for i=1, #array do
		arrCopy[i] = array[i]
	end
	return arrCopy
end

-- from http://snipplr.com/view/13092
-- Pads str to length len with char from right
LFRSH.lpad = function(str, len, char)
	if char == nil then char = ' ' end
	return str .. string.rep(char, len - #str)
end
--- Pads str to length len with char from left
LFRSH.rpad = function(str, len, char)
	if char == nil then char = ' ' end
	return string.rep(char, len - #str) .. str
end

LFRSH.SplitGold = function(gold)
	--if 
end

LFRSH.CountArray = function(a)
   local count = 0 
   for _ in pairs(a) do
      count = count + 1
   end
   return count
end