--[[
===============
=============== User interface creation    
===============
--]]

local LFRSH = LFRSatchelHelperGlobal

local CF = CreateFrame

local mainWidth, mainHeight = 370, 504
local innerWidth = 360

local leaderIconCoords  = {0.0000, 0.2617, 0.0000, 0.2617}
local tankIconCoords    = {0.0000, 0.2617, 0.2617, 0.5234}
local healerIconCoords  = {0.2617, 0.5234, 0.0000, 0.2617}
local damageIconCoords  = {0.2617, 0.5234, 0.2617, 0.5234}

LFRSH.instanceDropDown = {}
LFRSH.instanceDropDownSelected = nil
LFRSH.rolesDropDown = {}
LFRSH.rolesDropDownSelected = nil
LFRSH.notifyList = nil

LFRSH.MMIconFlashed = true
LFRSH.timeSinceLastFlash = 0

--[[
======
====== Main window buildup    
======
--]]

function LFRSH.CreateUI()
    local panel = CF("Frame", "LFRSatchelHelperFrame", UIParent, "UIPanelDialogTemplate")
    panel:Hide()
    panel:SetPoint("LEFT", 20, 0)
    panel:SetSize(mainWidth, mainHeight)
    panel:EnableMouse(true)
    panel:SetMovable(true)
    panel:SetToplevel(true)
    panel:RegisterForDrag("LeftButton")
    panel:HookScript("OnDragStart", function(self) self:StartMoving() end)
    panel:HookScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
    panel:HookScript("OnShow", function() LFRSH.DisableDungeonRows() PlaySound("igCharacterInfoOpen") end)
    panel:HookScript("OnHide", function() PlaySound("igCharacterInfoClose") end)

    panel.title:SetText("LFRSatchelHelper")

    -- Create the frames for each tab
    LFRSH.CreateTabRaids(panel)
    LFRSH.CreateTabDungeons(panel)
    LFRSH.CreateTabOptions(panel)
    --LFRSH.CreateTabStatistics(panel)
	
    -- Give main frame tab buttons to switch tabs NAME HAS TO BE Tab1, blizz code loops through these via framename.."Tab"..i !!!!!!!
    local btnTab1 = CF("Button", "$parentTab1", panel, "CharacterFrameTabButtonTemplate")
    btnTab1:SetPoint("CENTER", "LFRSatchelHelperFrame", "BOTTOMLEFT", 60, -10)
    btnTab1:SetText("LFR")
    btnTab1:SetID(1)
    btnTab1:HookScript("OnClick", function() PanelTemplates_SetTab(LFRSH.panel, 1) _G["LFRSH_Tab1"]:Show() _G["LFRSH_Tab2"]:Hide() _G["LFRSH_Tab3"]:Hide() --[[_G["LFRSH_Tab4"]:Hide()]] end)
    local btnTab2 = CF("Button", "$parentTab2", panel, "CharacterFrameTabButtonTemplate")
    btnTab2:SetPoint("LEFT", "$parentTab1", "RIGHT", -16, 0)
    btnTab2:SetText("Dungeons")
    btnTab2:SetID(2)
    btnTab2:HookScript("OnClick", function() PanelTemplates_SetTab(LFRSH.panel, 2) _G["LFRSH_Tab2"]:Show() _G["LFRSH_Tab1"]:Hide() _G["LFRSH_Tab3"]:Hide() --[[_G["LFRSH_Tab4"]:Hide()]] end)
    local btnTab3 = CF("Button", "$parentTab3", panel, "CharacterFrameTabButtonTemplate")
    btnTab3:SetPoint("LEFT", "$parentTab2", "RIGHT", -16, 0)
    btnTab3:SetText("Options")
    btnTab3:SetID(3)
    btnTab3:HookScript("OnClick", function() PanelTemplates_SetTab(LFRSH.panel, 3) _G["LFRSH_Tab3"]:Show() _G["LFRSH_Tab1"]:Hide() _G["LFRSH_Tab2"]:Hide() --[[_G["LFRSH_Tab4"]:Hide()]] end)    
	--[[local btnTab4 = CF("Button", "$parentTab4", panel, "CharacterFrameTabButtonTemplate")
    btnTab4:SetPoint("LEFT", "$parentTab3", "RIGHT", -16, 0)
    btnTab4:SetText("Statistics")
    btnTab4:SetID(4)
    btnTab4:HookScript("OnClick", function() PanelTemplates_SetTab(LFRSH.panel, 4) _G["LFRSH_Tab4"]:Show() _G["LFRSH_Tab1"]:Hide() _G["LFRSH_Tab2"]:Hide() _G["LFRSH_Tab3"]:Hide() end)    	
    --]]
	return panel
end

-- create tab 1 LFR frame
function LFRSH.CreateTabRaids(parentFrame)
	local bgFilePath = "Interface\\LFGFrame\\UI-LFG-BACKGROUND-HELLFIRECITADELRAID"
	local header, body = LFRSH.CreateTabFrame("LFRSH_Tab1", parentFrame, "LFR Wings", bgFilePath, true)
    
    -- List the raids
    for i = 1, #LFRSH.lfrIDs do
        local distanceToTop = -0.7 + i    
        LFRSH.CreateFrameRow(i, LFRSH.lfrIDs[i][1], body, innerWidth, distanceToTop*-30, "LFR")
    end         
end

-- create tab 2 Dungeon frame
function LFRSH.CreateTabDungeons(parentFrame)
	local bgFilePath = "Interface\\LFGFrame\\UI-LFG-BACKGROUND-BLACKROCKSPIRE"
	local header, body = LFRSH.CreateTabFrame("LFRSH_Tab2", parentFrame, "Dungeons", bgFilePath, true)

    -- List the dungeons
    for i = 1, #LFRSH.dungeonIDs do
        local distanceToTop = -0.7 + i    
        LFRSH.CreateFrameRow(i, LFRSH.dungeonIDs[i][1], body, innerWidth, distanceToTop*-30, "Dungeons")
    end
end

-- create tab 3 Options frame
function LFRSH.CreateTabOptions(parentFrame)
	local bgFilePath = "Interface\\LFGFrame\\UI-LFG-BACKGROUND-UTGARDEPINNACLE"
	local header, body = LFRSH.CreateTabFrame("LFRSH_Tab3", parentFrame, "Options", bgFilePath, false)	
	
	-- show chat messages option
	local showChat = LFRSH.CreateOptionsRow("ShowChat", body, header, 30)
	showChat.label:SetText("Show chat messages")
	local cbxShowChat = CF("CheckButton", "$parentCheckButton", showChat, "UICheckButtonTemplate")
	cbxShowChat:SetPoint("RIGHT", -20, 0)
	cbxShowChat:HookScript("OnClick", function(self) 
			if self:GetChecked() then 
				LFRSatchelHelperSettings["ShowChatMessages"] = true 
			else 
				LFRSatchelHelperSettings["ShowChatMessages"] = false 
			end
		end)
	-- set scan interval option
	local scanInterval = LFRSH.CreateOptionsRow("ScanInterval", body, showChat, 30)
	scanInterval.label:SetText("Scan interval in seconds")
	local ebScanInterval = CF("EditBox", "$parentEditBox", scanInterval, "InputBoxTemplate")
	ebScanInterval:SetSize(60, 20)
	ebScanInterval:SetPoint("RIGHT", -20, 0)
	ebScanInterval:SetJustifyH("RIGHT")
	ebScanInterval:SetTextInsets(3, 4, 0, 0)
	ebScanInterval:SetAutoFocus(false)
	ebScanInterval:SetNumeric(true)	
	ebScanInterval:HookScript("OnMouseDown", function(self) self:SetFocus() self:HighlightText() end )
	ebScanInterval:HookScript("OnEnterPressed", function(self) 
			local number = self:GetNumber()
			if number > 0 then
				LFRSatchelHelperSettings["ScanInterval"] = number
				LFRSH.print("Scan interval set to " .. LFRSH.FormatTime(number)) 
			else
				self:SetText(tostring(LFRSatchelHelperSettings["ScanInterval"]))
			end
			ebScanInterval:ClearFocus()
		end )
	-- set sound warning option
	local soundWarning = LFRSH.CreateOptionsRow("NotifySound", body, scanInterval, 30)
	soundWarning.label:SetText("Play notification sound")
	local cbxSoundWarning = CF("CheckButton", "$parentCheckButton", soundWarning, "UICheckButtonTemplate")
	cbxSoundWarning:SetPoint("RIGHT", -20, 0)
	cbxSoundWarning:HookScript("OnClick", function(self) 
			if self:GetChecked() then 
				LFRSatchelHelperSettings["NotifySound"] = true 
			else 
				LFRSatchelHelperSettings["NotifySound"] = false 
			end
		end)	
	-- set popup option TBI
	--[[local popupWarning = LFRSH.CreateOptionsRow("PopupWarning", body, soundWarning, 30)
	popupWarning.label:SetText("Popup notification warning")
	local cbxPopupWarning = CF("CheckButton", "$parentCheckButton", popupWarning, "UICheckButtonTemplate")
	cbxPopupWarning:SetPoint("RIGHT", -20, 0)
	cbxPopupWarning:HookScript("OnClick", function(self) 
			if self:GetChecked() then 
				LFRSatchelHelperSettings["PopupWarning"] = true 
			else 
				LFRSatchelHelperSettings["PopupWarning"] = false 
			end
		end)--]]		
		
		
	-- give user option to get notified when certain instances have a satchel for a specific role
	local notifyInstances = LFRSH.CreateOptionsRow("NotifyInstances", body, soundWarning, 150)
	notifyInstances.label:SetText("Add an instance-role combination for notifications")
	notifyInstances.label:SetPoint("TOPLEFT", 20, 0)

	LFRSH.instanceDropDown = LFRSH.CreateDropDownMenu(LFRSH.instanceDropDown, "$parentInstanceDropDown", notifyInstances, LFRSH.InitInstancesDropDown, 140)
	LFRSH.instanceDropDown:SetPoint("TOPLEFT", notifyInstances.label, "BOTTOMLEFT", -17, 0) 

	LFRSH.rolesDropDown = LFRSH.CreateDropDownMenu(LFRSH.rolesDropDown, "$parentRolesDropDown", notifyInstances, LFRSH.InitRolesDropDown, 80)
	LFRSH.rolesDropDown:SetPoint("LEFT", LFRSH.instanceDropDown, "RIGHT", -25, 0) 
	
	local addNotification = CF("Button", "$parentAdd", notifyInstances, "UIPanelButtonTemplate")
	addNotification:SetPoint("LEFT", LFRSH.rolesDropDown, "RIGHT", -12, 3)
	addNotification:SetSize(70, 25)
	addNotification:SetText("Add")
	addNotification:HookScript("OnClick", function(self)
			if self:GetText() == "Add" then
				-- add notification to settings array and show it in the scrollframe
				LFRSH.AddNotification(LFRSH.instanceDropDownSelected, LFRSH.rolesDropDownSelected)
			else
				-- remove notification from settings array and from the scrollframe
				LFRSH.RemoveNotification(LFRSH.instanceDropDownSelected, LFRSH.rolesDropDownSelected)
				self:SetText("Add")
			end
			-- reset dropdown selections
			--LFRSH.instanceDropDownSelected = nil
			--UIDropDownMenu_SetText(LFRSH.instanceDropDown, "") -- no longer reset this listbox after adding/removing for usability reasions
			LFRSH.rolesDropDownSelected = nil			
			UIDropDownMenu_SetText(LFRSH.rolesDropDown, "")
		end)
		
	local sf = CF("ScrollFrame", "$parentScrollFrame", notifyInstances, "InputScrollFrameTemplate")
	sf:SetPoint("TOPLEFT", LFRSH.instanceDropDown, "BOTTOMLEFT", 20, -5)
	sf:SetSize(innerWidth - 50, 80)
	sf.CharCount:Hide()
	LFRSH.notifyList = sf.EditBox
	LFRSH.notifyList:Disable()
	LFRSH.notifyList:SetWidth(sf:GetWidth())
	LFRSH.notifyList:SetMaxLetters(0)
	
	LFRSH.notifyList.Add = function(text) 
		LFRSH.notifyList:SetText(LFRSH.notifyList:GetText().."\n"..text)
	end
	LFRSH.notifyList.Clear = function() 
		LFRSH.notifyList:SetText("")
	end	
end

-- create tab 4 Statistics frame
function LFRSH.CreateTabStatistics(parentFrame)
	local bgFilePath = "Interface\\LFGFrame\\UI-LFG-BACKGROUND-ZULFARAK"
	local header, body = LFRSH.CreateTabFrame("LFRSH_Tab4", parentFrame, "Statistics", bgFilePath, false)
	
end

-- create a tab frame with a header and body frame
function LFRSH.CreateTabFrame(name, parentFrame, labelText, bgFilePath, includeIcons)
    local tabFrame = CF("Frame", name, parentFrame)
    tabFrame:SetAllPoints()
    tabFrame:Hide()

	-- Header
    local panelHeader = LFRSH.CreateTabHeader(tabFrame, labelText, bgFilePath, includeIcons)

    -- Body
    local panelBody = CF("Frame", "$parent_Body", tabFrame)
    panelBody:SetPoint("TOP", panelHeader:GetName(), 0, -80)
    panelBody:SetSize(innerWidth, 410)

	return panelHeader, panelBody
end

-- create the header frame for tabs
function LFRSH.CreateTabHeader(parentFrame, labelText, bgFilePath, includeIcons)

    local headerFrame = CF("Frame", "$parent_Header", parentFrame)
    headerFrame:SetPoint("TOP", 0, -28)
    headerFrame:SetSize(innerWidth, 80) 
    local label = headerFrame:CreateFontString("$parentLabel", "OVERLAY", "GameFontNormal")
    label:SetPoint("LEFT", 20, 0)
    label:SetText(labelText) 
	
    local backdrop = {
        -- path to the background texture 
        bgFile = bgFilePath,  
        -- true to repeat the background texture to fill the frame, false to scale it
        tile = false,
        -- distance from the edges of the frame to those of the background texture (in pixels)
        insets = {
            left = 5,
            right = 2,
            top = 0,
            bottom = 0
        }
    }    
    headerFrame:SetBackdrop(backdrop)	
		
	if includeIcons then
		local leaderIcon = headerFrame:CreateTexture("$parentLeaderIcon", "OVERLAY")
		leaderIcon:SetSize(40, 40)
		leaderIcon:SetPoint("LEFT", (innerWidth / 2) -12, 0)
		leaderIcon:SetTexture("Interface\\LFGFrame\\UI-LFG-ICON-ROLES")
		leaderIcon:SetTexCoord(unpack(leaderIconCoords))
		local tankIcon = headerFrame:CreateTexture("$parentTankIcon", "OVERLAY")
		tankIcon:SetSize(40, 40)
		tankIcon:SetPoint("RIGHT", "$parentLeaderIcon", 40, 0)
		tankIcon:SetTexture("Interface\\LFGFrame\\UI-LFG-ICON-ROLES")
		tankIcon:SetTexCoord(unpack(tankIconCoords))
		local healerIcon = headerFrame:CreateTexture("$parentHealerIcon", "OVERLAY")
		healerIcon:SetSize(40, 40)
		healerIcon:SetPoint("RIGHT", "$parentTankIcon", 40, 0)
		healerIcon:SetTexture("Interface\\LFGFrame\\UI-LFG-ICON-ROLES")
		healerIcon:SetTexCoord(unpack(healerIconCoords))
		local damageIcon = headerFrame:CreateTexture("$parentDamageIcon", "OVERLAY")
		damageIcon:SetSize(40, 40)
		damageIcon:SetPoint("RIGHT", "$parentHealerIcon", 40, 0)
		damageIcon:SetTexture("Interface\\LFGFrame\\UI-LFG-ICON-ROLES")
		damageIcon:SetTexCoord(unpack(damageIconCoords))
	end
    return headerFrame
end 

-- returns the row frame for a single instance
function LFRSH.CreateFrameRow(name, text, parent, width, y, instanceType)
    local prefix = "LFRSH_RaidRow"
	local extraRewardText = "    Weekly reward!"
	if instanceType == "Dungeons" then
		prefix = "LFRSH_DungeonRow"
		extraRewardText = "    Daily reward!"
	end
	local f = CF("Frame", prefix .. name, parent, "LFRSH_Instance_Row")
    f:SetSize(width, 30)
    f:SetPoint("TOP", 0, y)
    f.label:SetText(text)
	f.extraReward:SetText(extraRewardText)
    f.leaderCB:SetPoint("LEFT", (width / 2) -10, 0) 

    if not LFRSH.canTank then
        f.tankCB:Disable(true)
    end

    if not LFRSH.canHeal then
        f.healerCB:Disable(true)
    end

    if not LFRSH.canDPS then
        f.damageCB:Disable(true)
    end
	
    f.btnQueue:HookScript("OnClick", function() LFRSH.HandleQueueing(instanceType, name) end)
   -- return f
end

-- create a row in the options menu tab, each row has a default label left
function LFRSH.CreateOptionsRow(name, parentFrame, frameRelativeTo, height)
	local row = CF("Frame", "$parent_"..name, parentFrame)
	row:SetSize(innerWidth, height)
	row:SetPoint("TOP", frameRelativeTo, "BOTTOM")
	row.label = row:CreateFontString("$parentLabel", "OVERLAY", "GameFontNormal")
	row.label:SetPoint("LEFT", 20, 0)
	
	return row
end

--[[
======
====== Dropdown lists
======
--]]

-- create a dropdown list, function used by all dropdowns
function LFRSH.CreateDropDownMenu(object, name, parentFrame, initFunc, width)
	object = CreateFrame("FRAME", name, parentFrame, "UIDropDownMenuTemplate")
	UIDropDownMenu_SetWidth(object, width)
	
	-- Create and bind the initialization function to the dropdown menu
	UIDropDownMenu_Initialize(object, function(self, level, menuList)
		initFunc(self, level)	
	end)

	return object
end

-- Initialize the instances dropdown
function LFRSH.InitInstancesDropDown(self, level)
	local info = UIDropDownMenu_CreateInfo()
	info.func = LFRSH.InstanceDropDownClicked
	
	info.text, info.isTitle = "LFR", true
	UIDropDownMenu_AddButton(info, level)
	
	for i=1, #LFRSH.lfrIDs do
		info.text, info.arg1, info.arg2, info.isTitle, info.disabled = LFRSH.lfrIDs[i][1], LFRSH.lfrIDs[i][1], LFRSH.lfrIDs[i][2] , false, false
		UIDropDownMenu_AddButton(info, level)
	end	

	info.text, info.isTitle = "Dungeons", true
	UIDropDownMenu_AddButton(info, level)	

	for i=1, #LFRSH.dungeonIDs do
		info.text, info.arg1, info.arg2, info.isTitle, info.disabled = LFRSH.dungeonIDs[i][1], LFRSH.dungeonIDs[i][1], LFRSH.dungeonIDs[i][2] , false, false
		UIDropDownMenu_AddButton(info, level)
	end
	
end

-- Handle clicking on an instances dropdown item
function LFRSH.InstanceDropDownClicked(self, text, id, checked)
	LFRSH.instanceDropDownSelected = id
	UIDropDownMenu_SetText(LFRSH.instanceDropDown, text)
	LFRSH.CheckAlreadyAdded()
end

-- Initialize the roles dropdown
function LFRSH.InitRolesDropDown(self, level)
	local info = UIDropDownMenu_CreateInfo()
	info.func = LFRSH.RolesDropDownClicked
	
	info.text, info.arg1, info.arg2 = "Tank", "Tank", 1
	UIDropDownMenu_AddButton(info, level)
	info.text, info.arg1, info.arg2 = "Healer", "Healer", 2
	UIDropDownMenu_AddButton(info, level)
	info.text, info.arg1, info.arg2 = "Damage", "Damage", 3
	UIDropDownMenu_AddButton(info, level)	
end

-- Handle the clicking on a roles dropdown item
function LFRSH.RolesDropDownClicked(self, text, id, checked)
	LFRSH.rolesDropDownSelected = id
	UIDropDownMenu_SetText(LFRSH.rolesDropDown, text)
	LFRSH.CheckAlreadyAdded()
end

--[[
======
====== Minimap button
======
--]]

-- Position the minimap button around the minimap
function LFRSH.MinimapReposition()
    LFRSatchelHelper_MinimapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(LFRSatchelHelperSettings["MinimapButtonPos"])),(80*sin(LFRSatchelHelperSettings["MinimapButtonPos"]))-52);
end

-- Handle dragging the minimap button
function LFRSatchelHelper_MinimapButtonDrag()
    local xpos,ypos = GetCursorPosition();
    local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom();

    xpos = xmin-xpos/UIParent:GetScale()+70;
    ypos = ypos/UIParent:GetScale()-ymin-70;
    LFRSatchelHelperSettings["MinimapButtonPos"] = math.deg(math.atan2(ypos,xpos));
    LFRSH.MinimapReposition();
end

-- Handle click events on the button
function LFRSatchelHelper_MinimapButtonClick(Button)
    -- shift left click opens gui
    if Button == "LeftButton" and IsShiftKeyDown() then
        LFRSH.chatMsgPrinted = false;
        LFRSH.isManualScan = true
		-- force the update random info event (to get the latest information) after which we will do our scan
        RequestLFDPlayerLockInfo(); 

		LFRSH.ToggleQueueButtons()
		
		LFRSH.panel:Show();
       
    -- shift right click ..'.
    elseif Button == "RightButton" and IsShiftKeyDown() then

    -- leftclicking minimap icon is the same as typing /lfs scan
    elseif Button == "LeftButton" or Button == nil then           
        LFRSH.chatMsgPrinted = false;
        LFRSH.isManualScan = true
		-- force the update random info event (to get the latest information) after which we will do our scan
        RequestLFDPlayerLockInfo();
    elseif Button == "RightButton" then
        if LFRSH.autoScan then
            LFRSH.autoScan = false;
            LFRSH.print("Autscan disabled");
			SetDesaturation(_G["LFRSatchelHelper_MinimapButton_Icon"], false)
        else
            LFRSH.autoScan = true;
            LFRSH.timeSinceLastScan = 0;
            LFRSH.print("Autoscan enabled")
        end        
    end
end

-- 'blink' the minimap icon when autoscan is enabled
function LFRSatchelHelper_MinimapButtonUpdate(self, elapsed)
	if LFRSH.autoScan then
		LFRSH.timeSinceLastFlash = LFRSH.timeSinceLastFlash + elapsed
		if LFRSH.timeSinceLastFlash > 0.5 then	
			if LFRSH.MMIconFlashed then
				--_G["LFRSatchelHelper_MinimapButton_IconOverlay"]:Hide()
				SetDesaturation(_G["LFRSatchelHelper_MinimapButton_Icon"], false)
				LFRSH.MMIconFlashed = false
			else
				--_G["LFRSatchelHelper_MinimapButton_IconOverlay"]:Show()
				SetDesaturation(_G["LFRSatchelHelper_MinimapButton_Icon"], true)
				LFRSH.MMIconFlashed = true
			end
			LFRSH.timeSinceLastFlash = 0
		end
	end
end

--[[
======
====== Functions that change the UI after load
======
--]]

-- disable the rows for the dungeons that are not currently available
function LFRSH.DisableDungeonRows()

    -- cycle through all dungeons
    for i = 1, #LFRSH.dungeonIDs do
        local available = GetLFGDungeonRewardInfo(LFRSH.dungeonIDs[i][2], 1)

        if not available then
            _G["LFRSH_DungeonRow" .. i .. "Label"]:SetTextColor(0.4117, 0.4117, 0.4117, 1)
            _G["LFRSH_DungeonRow" .. i .. "LeaderCB"]:Disable(true)
            _G["LFRSH_DungeonRow" .. i .. "TankCB"]:Disable(true)
            _G["LFRSH_DungeonRow" .. i .. "HealerCB"]:Disable(true)
            _G["LFRSH_DungeonRow" .. i .. "DamageCB"]:Disable(true)
            _G["LFRSH_DungeonRow" .. i .. "Queue"]:Disable(true)
        end
    end
end

-- check the checkboxes in the gui based on the results of the scan
function LFRSH.UpdateLFRGUI(scanResults, weekly)
    -- update checkboxes if we have results
	if scanResults ~= nil then
        for i = 1, #scanResults do
            local tank, healer, damage = unpack(scanResults[i])
            -- uncheck the checkboxes
            _G["LFRSH_RaidRow"..i.."TankCB"]:SetChecked(false)
            _G["LFRSH_RaidRow"..i.."HealerCB"]:SetChecked(false)
            _G["LFRSH_RaidRow"..i.."DamageCB"]:SetChecked(false)

            if tank then
                _G["LFRSH_RaidRow"..i.."TankCB"]:SetChecked(true)
            end
            if healer then
                _G["LFRSH_RaidRow"..i.."HealerCB"]:SetChecked(true)
            end
            if damage then
                _G["LFRSH_RaidRow"..i.."DamageCB"]:SetChecked(true)
            end        
        end
	else
		for i = 1, #LFRSH.lfrIDs do
            _G["LFRSH_RaidRow"..i.."TankCB"]:SetChecked(false)
            _G["LFRSH_RaidRow"..i.."HealerCB"]:SetChecked(false)
            _G["LFRSH_RaidRow"..i.."DamageCB"]:SetChecked(false)		
		end		
		
    end 
	-- update weekly rewards
	for i = 1, #weekly do
		if not weekly[i] then
			_G["LFRSH_RaidRow"..i.."ExtraReward"]:Show()    
		else
			_G["LFRSH_RaidRow"..i.."ExtraReward"]:Hide()
		end
	end	
end

-- check the checkboxes in the dungeon gui based on scan results
function LFRSH.UpdateDungeonGUI(scanResults, daily)
    -- update checkboxes if we have results
	if scanResults ~= nil then
        for i = 1, #scanResults do
            local tank, healer, damage = unpack(scanResults[i])
            -- uncheck the checkboxes
            _G["LFRSH_DungeonRow"..i.."TankCB"]:SetChecked(false)
            _G["LFRSH_DungeonRow"..i.."HealerCB"]:SetChecked(false)
            _G["LFRSH_DungeonRow"..i.."DamageCB"]:SetChecked(false)

            if tank then
                _G["LFRSH_DungeonRow"..i.."TankCB"]:SetChecked(true)
            end
            if healer then
                _G["LFRSH_DungeonRow"..i.."HealerCB"]:SetChecked(true)
            end
            if damage then
                _G["LFRSH_DungeonRow"..i.."DamageCB"]:SetChecked(true)
            end         
        end
	else
		for i = 1, #LFRSH.dungeonIDs do
            _G["LFRSH_DungeonRow"..i.."TankCB"]:SetChecked(false)
            _G["LFRSH_DungeonRow"..i.."HealerCB"]:SetChecked(false)
            _G["LFRSH_DungeonRow"..i.."DamageCB"]:SetChecked(false)		
		end
		
    end	
	--update daily rewards
	for i = 1, #daily do
		if not daily[i] then
			_G["LFRSH_DungeonRow"..i.."ExtraReward"]:Show()    
		else
			_G["LFRSH_DungeonRow"..i.."ExtraReward"]:Hide()
		end

		-- timewalking dungeons don't have a daily reward
		if i ~= 1 then
			_G["LFRSH_DungeonRow"..i.."ExtraReward"]:Hide()
		end 
	end	
end

-- Check if a instance - role combination is already present in the notification list
function LFRSH.CheckAlreadyAdded()
	local instanceID, roleID = LFRSH.instanceDropDownSelected, LFRSH.rolesDropDownSelected
	if instanceID and roleID then
		if LFRSatchelHelperSettings["NotifyInstances"][instanceID][roleID] then
			-- added this combination already => change button to remove it
			_G["LFRSH_Tab3_Body_NotifyInstancesAdd"]:SetText("Remove")
		else
			-- hasn't been added yet => update button text
			_G["LFRSH_Tab3_Body_NotifyInstancesAdd"]:SetText("Add")
		end
	end
end

-- update the data in the frame that shows the notified instances
function LFRSH.UpdateNotificationsFrame()
	LFRSH.notifyList.Clear()
	-- we have to use lfrIDs and dungeonIDs to get them in the right order...
	for i=1, #LFRSH.lfrIDs do
		local instanceName = LFRSH.lfrIDs[i][1]
		local instanceID = LFRSH.lfrIDs[i][2]
		local rolesText = ""
		local roles = LFRSatchelHelperSettings["NotifyInstances"][instanceID]
		for roleID,notifyRole in pairs(roles) do
			if notifyRole then
				local roleName = LFRSH.quickLookupRoles[roleID][1]
				rolesText = rolesText .. "\124c" .. LFRSH.quickLookupRoles[roleID][2] ..  roleName .. "\124r "
			end
		end
		if rolesText ~= "" then
			instanceName = LFRSH.lpad(instanceName, 25, "_")
			LFRSH.notifyList.Add("\124cFFFFFF00"..instanceName .. "\124r" .. rolesText)
		end		
	end
	
	for i=1, #LFRSH.dungeonIDs do
		local instanceName = LFRSH.dungeonIDs[i][1]
		local instanceID = LFRSH.dungeonIDs[i][2]
		local rolesText = ""
		local roles = LFRSatchelHelperSettings["NotifyInstances"][instanceID]
		for roleID,notifyRole in pairs(roles) do
			if notifyRole then
				local roleName = LFRSH.quickLookupRoles[roleID][1]
				rolesText = rolesText .. "\124c" .. LFRSH.quickLookupRoles[roleID][2] .. roleName .. "\124r "
			end
		end
		if rolesText ~= "" then
			instanceName = LFRSH.lpad(instanceName, 25, "_")
			LFRSH.notifyList.Add("\124cFFFFFF00"..instanceName .. "\124r" .. rolesText)
		end		
	end
	
end

-- toggle queue buttons
function LFRSH.ToggleQueueButtons()	
	local index = nil
	if LFRSH.CountArray(LFGQueuedForList[3]) == 0 or LFRSH.CountArray(LFGQueuedForList[1]) == 0 then
		for typeID, typeData in pairs(LFRSH.quickLookupIDs) do
			for iID, iData in pairs(typeData) do 
				if typeID == 1 then
					_G["LFRSH_RaidRow"..iData[1].."Queue"]:Enable()
				else
					_G["LFRSH_DungeonRow"..iData[1].."Queue"]:Enable()
				end
			end
		end
	end
	
	for instanceID, isQueued in pairs(LFGQueuedForList[3]) do
		index = LFRSH.QuickLookup(instanceID) 
		if index ~= nil then		
			if isQueued then
				_G["LFRSH_RaidRow"..index[1].."Queue"]:Disable()
			else
				_G["LFRSH_RaidRow"..index[1].."Queue"]:Enable()
			end
		end
	end
	for instanceID, isQueued in pairs(LFGQueuedForList[1]) do
		index = LFRSH.QuickLookup(instanceID)
		if index ~= nil then
			if isQueued then
				_G["LFRSH_DungeonRow"..index[1].."Queue"]:Disable()
			else
				_G["LFRSH_DungeonRow"..index[1].."Queue"]:Enable()
			end		
		end
	end	
	LFRSH.DisableDungeonRows()
end

